#include <mbed.h>

// SPI lábak beállítása RP2040-on
#define MOSI_PIN p3
#define MISO_PIN p4
#define SCLK_PIN p2
#define CS_PIN p5

mbed::SPI spi(MOSI_PIN, MISO_PIN, SCLK_PIN); // SPI inicializáció
mbed::DigitalOut cs(CS_PIN);                 // Chip Select pin

const unsigned char led1[] = {0xFF, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0xFF}; // H
const unsigned char led2[] = {0x1F, 0x60, 0x80, 0x40, 0x40, 0x80, 0x60, 0x1F}; // W

/// SPI adatok küldése
void SPI_Write2(unsigned char MSB, unsigned char LSB) {
  cs = 0;                       // CS Low
  spi.write(MSB);               // Küldés
  spi.write(LSB);               // Küldés
  cs = 1;                       // CS High
}

/// MAX7219 inicializálása
void Init_MAX7219() {
  SPI_Write2(0x09, 0x00);       // Dekódolás kikapcsolva
  SPI_Write2(0x0A, 0x08);       // Közepes fényerő
  SPI_Write2(0x0B, 0x07);       // Scan limit = 7
  SPI_Write2(0x0C, 0x01);       // Normál működés
  SPI_Write2(0x0F, 0x0F);       // Tesztmód engedélyezése
  delay(500);                   // 500 ms késleltetés
  for (int i=1; i<=8; i++) {
    SPI_Write2(i, 0x00);        // Sorok törlése
  }
  SPI_Write2(0x0F, 0x00);       // Tesztmód kikapcsolása
  delay(500);                   // 500 ms késleltetés
}

void setup() {
  cs = 1;                       // CS kezdetben High
  spi.format(8, 0);             // 8-bites formátum, 0 mód
  spi.frequency(1000000);       // 1 MHz SCLK frekvencia
  Init_MAX7219();               // MAX7219 inicializálása
}

void loop() {
  for (int i = 1; i < 9; i++) { // Első karakter kiírása
    SPI_Write2(i, led1[i - 1]);
  }
  delay(1000); // 1 másodperc késleltetés
  for (int i = 1; i < 9; i++) { // Második karakter kiírása
    SPI_Write2(i, led2[i - 1]);
  }
  delay(1000);                  // 1 másodperc késleltetés
}
