import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

# Adatok
frequencies = np.array([1, 4, 10, 20, 40])  # Mért SPI frekvenciák (MHz)
times = np.array([606, 360, 310, 294, 282])  # Mért végrehajtási idők (ms)

# Modellfüggvény: time = t0 + a / freq
def model(freq, t0, a):
    return t0 + a / freq

# Illesztés
params, covariance = curve_fit(model, frequencies, times)
t0, a = params

# Finomított X-tengely (SPI frekvenciák) 500 ponttal
smooth_frequencies = np.linspace(min(frequencies), max(frequencies), 500)
smooth_times = model(smooth_frequencies, t0, a)

# Nyomtatás az illesztett paraméterekhez
print(f"t0 (additív tag): {t0:.2f} ms")
print(f"a (arányossági konstans): {a:.2f}")

# Grafikon létrehozása
plt.figure(figsize=(10, 6))
plt.scatter(frequencies, times, color='red', label='Mért adatok')  # Eredeti adatok
plt.plot(smooth_frequencies, smooth_times, color='blue', label=f'Illesztett modell: time = {t0:.2f} + {a:.2f}/freq')
plt.xlabel('SPI Frekvencia (MHz)')
plt.ylabel('Végrehajtási Idő (ms)')
plt.title('SPI Frekvencia vs Végrehajtási Idő')
plt.grid(True)
plt.legend()
plt.show()
